@@ -35,10 +35,11 @@ public:
 		UPDATE_LED,
 		UPDATE_SETTING,
 		UPDATE_PLUG,
-		UPDATE_UNPLUG,
 		UPDATE_MEDIA,
-		UPDATE_STATE
-
+		UPDATE_STATE,
+		UPDATE_HARDWARE,
+		UPDATE_EXTENSION,
+		UPDATE_CONNECTOR
 	};
 	struct ParseResult {
 		ParseState parseState;
