@@ -9,6 +9,7 @@
 #include "IPSSelectionDlg.h"
 #include "utils.h"
 #include <algorithm>
+#include <map>
 #include <wx/wxprec.h>
 #include <wx/xrc/xmlres.h>
 #include <wx/button.h>
@@ -155,7 +156,6 @@ SessionPage::SessionPage(wxWindow* parent, openMSXController& controller)
 	m_hasCassettePort = true; // avoid UMR
 	m_cassetteControl = true; // see comments in OnMotorControl()
 	m_romTypeDialog.reset(new RomTypeDlg(wxGetTopLevelParent(this)));
-	GetRomTypes();
 	m_ipsDialog.reset(new IPSSelectionDlg(wxGetTopLevelParent(this)));
 	media[DISKA].reset(new MediaInfo(
 		*m_diskMenu[0], wxT("diska"), wxT("DiskAContents"),
@@ -188,7 +188,7 @@ SessionPage::SessionPage(wxWindow* parent, openMSXController& controller)
 		ConfigurationData::CD_NONE, ConfigurationData::CD_NONE,
 		m_hardDiskButton, HARDDISK, 0, 0));
 	for (auto& m : media) {
-		m->control.SetDropTarget(new SessionDropTarget(&m->control));
+		m->control.SetDropTarget(new SessionDropTarget(m.get(), this));
 	}
 
 	int autorecord;
@@ -286,7 +286,7 @@ void SessionPage::OnModeRecord(wxCommandEvent& event)
 		                     defaultpath, wxT(""), path,
 		                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 		if (filedlg.ShowModal() != wxID_OK) return;
-		cmd += wxT(" ") + utils::ConvertPath(filedlg.GetPath());
+		cmd += wxT(" ") + utils::ConvertPathForCommand(filedlg.GetPath());
 	}
 	m_controller.WriteCommand(cmd);
 }
@@ -334,14 +334,14 @@ void SessionPage::OnBrowseCartB(wxCommandEvent& event)
 void SessionPage::BrowseDisk(MediaInfo& m)
 {
 	BrowseMedia(m,
-		wxT("All known disk files|*.dsk;*.DSK;*.xsa;*.XSA;*.dmk;*.DMK;*.zip;*.ZIP;*.gz;*.GZ;*.di1;*.DI1;*.di2;*.DI2|Uncompressed disk files|*.dsk;*.DSK;*.xsa;*.XSA;*.di1;*.DI1;*.di2;*.DI2|Compressed files (*.zip;*.gz)|*.gz;*.GZ;*.zip;*.ZIP|All files|*.*||"),
+		wxT("All known disk files|*.dsk;*.DSK;*.xsa;*.XSA;*.dmk;*.DMK;*.zip;*.ZIP;*.gz;*.GZ;*.di1;*.DI1;*.di2;*.DI2;*.fd1;*.FD1;*.fd2;*.FD2|Uncompressed disk files|*.dsk;*.DSK;*.xsa;*.XSA;*.di1;*.DI1;*.di2;*.DI2;*.fd1;*.FD1;*.fd2;*.FD2|Compressed files (*.zip;*.gz)|*.gz;*.GZ;*.zip;*.ZIP|All files|*.*||"),
 		wxT("Select disk image"));
 }
 
 void SessionPage::BrowseCart(MediaInfo& m)
 {
 	BrowseMedia(m,
-		wxT("All known cartridge files|*.rom;*.ROM;*.ri;*.RI;*.zip;*.ZIP;*.gz;*.GZ|Uncompressed cartridge files|*.rom;*.ROM;*.ri;*.RI|Compressed files (*.zip;*.gz)|*.gz;*.GZ;*.zip;*.ZIP|All files|*.*||"),
+		wxT("All known cartridge files|*.rom;*.ROM;*.ri;*.RI;*.mx1;*.MX1;*.mx2;*.MX2;*.zip;*.ZIP;*.gz;*.GZ|Uncompressed cartridge files|*.rom;*.ROM;*.ri;*.RI;*.mx1;*.MX1;*.mx2;*.MX2|Compressed files (*.zip;*.gz)|*.gz;*.GZ;*.zip;*.ZIP|All files|*.*||"),
 		wxT("Select ROM image"));
 }
 
@@ -355,7 +355,7 @@ void SessionPage::OnBrowseCassette(wxCommandEvent& event)
 void SessionPage::OnBrowseHardDisk(wxCommandEvent& event)
 {
 	BrowseMedia(*media[HDD],
-		wxT("All known hard disk files|*.dsk;*.DSK;*.zip;*.ZIP;*.gz;*.GZ;|Uncompressed disk files|*.dsk;*.DSK;|Compressed files (*.zip;*.gz)|*.gz;*.GZ;*.zip;*.ZIP|All files|*.*||"),
+		wxT("All known hard disk files|*.dsk;*.DSK;*.zip;*.ZIP;*.gz;*.GZ;*.hdd;*.HDD;|Uncompressed disk files|*.dsk;*.DSK;*.hdd;*.HDD;|Compressed files (*.zip;*.gz)|*.gz;*.GZ;*.zip;*.ZIP|All files|*.*||"),
 		wxT("Select hard disk image"));
 }
 void SessionPage::BrowseMedia(MediaInfo& m, const wxString& path, const wxString title)
@@ -404,7 +404,7 @@ void SessionPage::ClickMediaCombo(wxCommandEvent& event, MediaInfo& m)
 void SessionPage::SetIpsList(MediaInfo& m, const wxArrayString& ips)
 {
 	m.ips = ips;
-	int count = m.ips.GetCount();
+	size_t count = m.ips.GetCount();
 	m.menu.SetLabel(m.ipsLabel, (count > 0)
 		? wxString::Format(wxT("Select IPS Patches (%d selected)"), count)
 		: wxString(wxT("Select IPS Patches (None selected)")));
@@ -431,8 +431,8 @@ static wxArrayString split(const wxString& str, wxString separator = wxT("::"))
 	wxArrayString result;
 	int pos = 0;
 	while (true) {
-		int pos2 = str.find(separator, pos);
-		if (pos2 == wxNOT_FOUND) return result;
+		size_t pos2 = str.find(separator, pos);
+		if (pos2 == wxString::npos) return result;
 		result.Add(str.Mid(pos, pos2 - pos));
 		pos = pos2 + 2;
 	}
@@ -462,21 +462,8 @@ void SessionPage::SetupHardware(bool initial, bool reset)
 		m_machineArray   = split(checkedMachines);
 		m_extensionArray = split(checkedExtensions);
 	} else {
-		m_machineArray.Clear();
-		m_extensionArray.Clear();
-		wxString sharepath;
-		config.GetParameter(ConfigurationData::CD_SHAREPATH, sharepath);
-		prepareExtensions(sharepath);
-		prepareMachines(sharepath);
-#ifdef __UNIX__
-		wxString personalShare = ::wxGetHomeDir() + wxT("/.openMSX/share");
-#else
-		TCHAR temp[MAX_PATH + 1];
-		SHGetSpecialFolderPath(0, temp, CSIDL_PERSONAL, 1);
-		wxString personalShare = wxString((const wxChar*)temp) + wxT("/openMSX/share");
-#endif
-		prepareExtensions(personalShare, true);
-		prepareMachines(personalShare, true);
+		prepareExtensions();
+		prepareMachines();
 	}
 	m_extensionArray.Sort(CompareCaseInsensitive);
 	m_machineArray.Sort(CompareCaseInsensitive);
@@ -504,69 +491,32 @@ void SessionPage::SetupHardware(bool initial, bool reset)
 	}
 }
 
-void SessionPage::prepareExtensions(const wxString& sharepath, bool optional)
+void SessionPage::prepareExtensions()
 {
-	wxString extPath = sharepath + wxT("/extensions");
-	if (!::wxDirExists(extPath)) {
-		if (!optional) {
-			wxMessageBox(wxT("Directory: ") + extPath +
-			             wxT(" does not exist"));
-		}
-		return;
-	}
-	wxDir sharedir(extPath);
-	wxString extension;
-	bool succes = sharedir.GetFirst(&extension);
-	while (succes) {
-		wxString fullPath = extPath + wxT("/") + extension;
-		wxString base;
-		if (extension.EndsWith(wxT(".xml"), &base)) {
-			if (::wxFileExists(fullPath)) {
-				if (m_extensionArray.Index(base) == wxNOT_FOUND) {
-					m_extensionArray.Add(base);
-				}
-			}
-		} else if (::wxFileExists(fullPath + wxT("/hardwareconfig.xml"))) {
-			if (m_extensionArray.Index(extension) == wxNOT_FOUND) {
-				m_extensionArray.Add(extension);
-			}
-		}
-		succes = sharedir.GetNext(&extension);
+	m_extensionArray.Clear();
+
+	wxString cmd;
+	ConfigurationData::instance().GetParameter(ConfigurationData::CD_EXECPATH, cmd);
+
+	wxArrayString output;
+	int code = wxExecute(cmd + wxT(" -bash -ext"), output);
+	if (code != -1) {
+		m_extensionArray = output;
 	}
 }
 
-void SessionPage::prepareMachines(const wxString& sharepath, bool optional)
+void SessionPage::prepareMachines()
 {
+	m_machineArray.Clear();
+
 	wxString cmd;
 	ConfigurationData::instance().GetParameter(ConfigurationData::CD_EXECPATH, cmd);
-	wxString machPath = sharepath + wxT("/machines");
-	if (!::wxDirExists(machPath)) {
-		if (!optional) {
-			wxMessageBox(wxT("Directory: ") + machPath +
-			             wxT(" does not exist"));
-		}
-		return;
+
+	wxArrayString output;
+	int code = wxExecute(cmd + wxT(" -bash -machine"), output);
+	if (code != -1) {
+		m_machineArray = output;
 	}
-	wxDir sharedir(machPath);
-	wxString machine;
-	bool succes = sharedir.GetFirst(&machine);
-	while (succes) {
-		wxString fullPath = machPath + wxT("/") + machine;
-		wxString base;
-		if (machine.EndsWith(wxT(".xml"), &base)) {
-			if (::wxFileExists(fullPath)) {
-				if (m_machineArray.Index(base) == wxNOT_FOUND) {
-					m_machineArray.Add(base);
-				}
-			}
-		} else if (::wxFileExists(fullPath + wxT("/hardwareconfig.xml"))) {
-			if (m_machineArray.Index(machine) == wxNOT_FOUND) {
-				m_machineArray.Add(machine);
-			}
-		}
-		succes = sharedir.GetNext(&machine);
-	}
-	m_machineList->SetSelection(0);
 }
 
 void SessionPage::HandleFocusChange(wxWindow* oldFocus, wxWindow* newFocus)
@@ -601,14 +551,14 @@ void SessionPage::insertMedia(MediaInfo& m)
 	m.lastContents = contents;
 	wxString cmd = m.deviceName + wxT(" ");
 	if (!contents.IsEmpty()) {
-		cmd += utils::ConvertPath(contents);
+		cmd += utils::ConvertPathForCommand(contents);
 		if (m.mediaType == CARTRIDGE) {
 			if (!m.type.IsEmpty()) {
 				cmd += wxT(" -romtype ") + m.type;
 			}
 		}
 		for (auto& ips : m.ips) {
-			cmd += wxT(" -ips ") + utils::ConvertPath(ips);
+			cmd += wxT(" -ips ") + utils::ConvertPathForCommand(ips);
 		}
 		AddHistory(m);
 	} else {
@@ -744,13 +694,7 @@ void SessionPage::RestoreHistory()
 				type,
 				split(ips, wxT(";;")))); // 2nd-level
 		}
-		if ((insertedMedia & m->mediaBits) &&
-#if wxVERSION_NUMBER < 2903
-				!m->control.IsEmpty()
-#else
-				!m->control.IsTextEmpty()
-#endif
-				) {
+		if ((insertedMedia & m->mediaBits) && !m->control.IsListEmpty()) {
 			m->control.SetSelection(0);
 			auto* h = static_cast<HistoryData*>(m->control.GetClientObject(0));
 			if (m->mediaType == CARTRIDGE) SetMapperType(*m, h->type);
@@ -931,49 +875,72 @@ void SessionPage::OnBrowseIps(wxCommandEvent& event)
 
 void SessionPage::GetRomTypes()
 {
-	// TODO get it from openMSX
-	// NOTE: this list is incomplete! REALLY GET IT FROM OPENMSX!
-	SetupRomType(wxT(""), wxT("Autodetect type"));
-	SetupRomType(wxT("AUTO"), wxT("Autodetect type"));
-	SetupRomType(wxT("Page0"), wxT("Plain 16 kB Page 0"));
-	SetupRomType(wxT("Page1"), wxT("Plain 16 kB Page 1"));
-	SetupRomType(wxT("Page2"), wxT("Plain 16 kB Page 2 (Basic)"));
-	SetupRomType(wxT("Normal8000"), wxT("Plain 16 kB Page 2 (Basic)"));
-	SetupRomType(wxT("Page3"), wxT("Plain 16 kB Page 3"));
-	SetupRomType(wxT("Page01"), wxT("Plain 32 kB Page 0-1"));
-	SetupRomType(wxT("Page12"), wxT("Plain 32 kB Page 1-2"));
-	SetupRomType(wxT("Page23"), wxT("Plain 32 kB Page 2-3"));
-	SetupRomType(wxT("Page012"), wxT("Plain 48 kB Page 0-2"));
-	SetupRomType(wxT("Page123"), wxT("Plain 48 kB Page 1-3"));
-	SetupRomType(wxT("Page0123"), wxT("Plain 64 kB"));
-	SetupRomType(wxT("Mirrored"), wxT("Plain (Any size)"));
-	SetupRomType(wxT("8kB"), wxT("Generic 8kB"));
-	SetupRomType(wxT("MSXDOS2"), wxT("MSX-DOS 2"));
-	SetupRomType(wxT("Konami"), wxT("Konami without SCC"));
-	SetupRomType(wxT("KonamiSCC"), wxT("Konami with SCC"));
-	SetupRomType(wxT("ASCII8"), wxT("Ascii 8kB"));
-	SetupRomType(wxT("ASCII16"), wxT("Ascii 16kB"));
-	SetupRomType(wxT("Padial8"), wxT("Padial 8kB"));
-	SetupRomType(wxT("Padial16"), wxT("Padial 16kB"));
-	SetupRomType(wxT("R-Type"), wxT("R-Type"));
-	SetupRomType(wxT("CrossBlaim"), wxT("Cross Blaim"));
-	SetupRomType(wxT("MSX-AUDIO"), wxT("MSX-Audio"));
-	SetupRomType(wxT("HarryFox"), wxT("Harry Fox"));
-	SetupRomType(wxT("Halnote"), wxT("Halnote"));
-	SetupRomType(wxT("Playball"), wxT("Playball"));
-	SetupRomType(wxT("Zemina80in1"), wxT("Zemina Multigame (80 in 1)"));
-	SetupRomType(wxT("Zemina90in1"), wxT("Zemina Multigame (90 in 1)"));
-	SetupRomType(wxT("Zemina126in1"), wxT("Zemina Multigame (126 in 1)"));
-	SetupRomType(wxT("HolyQuran"), wxT("Holy Qu'ran"));
-	SetupRomType(wxT("ASCII16SRAM2"), wxT("Ascii 16kB with SRAM"));
-	SetupRomType(wxT("ASCII8SRAM8"), wxT("Ascii 8kB with SRAM"));
-	SetupRomType(wxT("KoeiSRAM8"), wxT("Koei with 8kB SRAM"));
-	SetupRomType(wxT("KoeiSRAM32"), wxT("Koei with 32kB SRAM"));
-	SetupRomType(wxT("Wizardry"), wxT("Wizardry"));
-	SetupRomType(wxT("GameMaster2"), wxT("Konami Game Master 2"));
-	SetupRomType(wxT("Synthesizer"), wxT("Konami Synthesizer"));
-	SetupRomType(wxT("Majutsushi"), wxT("Hai no Majutsushi"));
-	SetupRomType(wxT("KeyboardMaster"), wxT("Konami Keyboard Master"));
+	// TODO get the descriptions from openMSX
+
+	wxString cmd;
+	ConfigurationData::instance().GetParameter(ConfigurationData::CD_EXECPATH, cmd);
+
+	wxArrayString output;
+	int code = wxExecute(cmd + wxT(" -bash -romtype"), output);
+	assert(code != -1);
+	if (code) {}; // avoid warning in build with asserts disabled
+
+	output.Sort(CompareCaseInsensitive);
+	output.Insert(wxT(""), 0);
+	output.Insert(wxT("AUTO"), 1);
+
+	std::map<std::string, std::string> descriptions = {
+		{"", "Autodetect type"},
+		{"AUTO", "Autodetect type"},
+		{"Page0", "Plain 16 kB Page 0"},
+		{"Page1", "Plain 16 kB Page 1"},
+		{"Page2", "Plain 16 kB Page 2 (Basic)"},
+		{"Normal8000", "Plain 16 kB Page 2 (Basic)"},
+		{"Page3", "Plain 16 kB Page 3"},
+		{"Page01", "Plain 32 kB Page 0-1"},
+		{"Page12", "Plain 32 kB Page 1-2"},
+		{"Page23", "Plain 32 kB Page 2-3"},
+		{"Page012", "Plain 48 kB Page 0-2"},
+		{"Page123", "Plain 48 kB Page 1-3"},
+		{"Page0123", "Plain 64 kB"},
+		{"Mirrored", "Plain (Any size)"},
+		{"8kB", "Generic 8kB"},
+		{"MSXDOS2", "MSX-DOS 2"},
+		{"Konami", "Konami without SCC"},
+		{"KonamiSCC", "Konami with SCC"},
+		{"ASCII8", "Ascii 8kB"},
+		{"ASCII16", "Ascii 16kB"},
+		{"Padial8", "Padial 8kB"},
+		{"Padial16", "Padial 16kB"},
+		{"R-Type", "R-Type"},
+		{"CrossBlaim", "Cross Blaim"},
+		{"MSX-AUDIO", "MSX-Audio"},
+		{"HarryFox", "Harry Fox"},
+		{"Halnote", "Halnote"},
+		{"Playball", "Playball"},
+		{"Zemina80in1", "Zemina Multigame (80 in 1)"},
+		{"Zemina90in1", "Zemina Multigame (90 in 1)"},
+		{"Zemina126in1", "Zemina Multigame (126 in 1)"},
+		{"HolyQuran", "Holy Qu'ran"},
+		{"ASCII16SRAM2", "Ascii 16kB with SRAM"},
+		{"ASCII8SRAM8", "Ascii 8kB with SRAM"},
+		{"KoeiSRAM8", "Koei with 8kB SRAM"},
+		{"KoeiSRAM32", "Koei with 32kB SRAM"},
+		{"Wizardry", "Wizardry"},
+		{"GameMaster2", "Konami Game Master 2"},
+		{"Synthesizer", "Konami Synthesizer"},
+		{"Majutsushi", "Hai no Majutsushi"},
+		{"KeyboardMaster", "Konami Keyboard Master"}
+	};
+
+	for (wxString& type : output) {
+		wxString description = type;
+		std::string stdType = std::string(type.mb_str());
+		if (descriptions.find(stdType) != descriptions.end()) {
+			description = wxString(descriptions[stdType].c_str(), wxConvUTF8);
+		}
+		SetupRomType(type, description);
+	}
 }
 
 void SessionPage::SetupRomType(wxString type, wxString fullname)
@@ -991,15 +958,17 @@ const wxArrayString& SessionPage::GetDetectedExtensions() const
 	return m_extensionArray;
 }
 
-SessionDropTarget::SessionDropTarget(wxComboBox* target)
+SessionDropTarget::SessionDropTarget(SessionPage::MediaInfo* target, SessionPage* sessionPage)
+	: m_target(target)
+	, m_sessionPage(sessionPage)
 {
-	m_target = target;
 }
 
 bool SessionDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
 {
 	if (!filenames.IsEmpty()) {
-		m_target->Append(filenames[0]); // just the first for starters
+		m_target->control.SetValue(filenames[0]); // just the first for starters
+		m_sessionPage->insertMediaClear(*m_target);
 	}
 	return true;
 }
